require "hump.camera"

function createBall(size,color,x,y)
	local ball = {}
	ball.color = color
	ball.body = love.physics.newBody(world, x, y, "dynamic")
	ball.shape = love.physics.newCircleShape(size)
	ball.fixture = love.physics.newFixture(ball.body, ball.shape, 1)
	ball.fixture:setRestitution(1.0)
	return ball
end

function createPlatform(length,size,x,y,rotation)
	local platform = {}
	platform.body = love.physics.newBody(world, x, y)
	platform.shape = love.physics.newRectangleShape(size,length)
	platform.body:setAngle(rotation)
	platform.fixture = love.physics.newFixture(platform.body, platform.shape)
	
	return platform	
end

function createWall(x,y,w,h)
	local wall = {}
	wall.body = love.physics.newBody(world, x/2, y/2)
	wall.shape = love.physics.newRectangleShape(w,h)
	wall.fixture = love.physics.newFixture(wall.body, wall.shape)
	return wall
end

function createWorldBoundaries(w,h)
	local walls = {} --Todo: figure out desired lengths.
	walls.wall1 = createWall(0,0,50,h)
	walls.wall2 = createWall(0,h,w,50)
	walls.wall3 = createWall(w-50,0,50,h)
	walls.wall4 = createWall(0,0,w,50)
	for k,wall in pairs(walls) do
		table.insert(objects.walls,wall)
	end
end

function love.load()
	--Useful constants.
	W = love.graphics.getWidth()
	H = love.graphics.getHeight()

	--Initialize physics.
	love.physics.setMeter(64)
	world = love.physics.newWorld(0, 9.81*64, true)

	objects = {}
	objects.balls = {}
	objects.platforms = {}
	objects.walls = {}

	createWorldBoundaries(2500,2500)
	
	player = createBall(20,{0,255,0},650/2,650/2)
	table.insert(objects.balls,player)
	enemy = createBall(20,{255,0,0},650/2+30,650/2)
	table.insert(objects.balls,enemy)

	--Now that the player has been given a position, set up the camera.

	table.insert(objects.platforms,createPlatform(100,10,500,500,1.6293))	

	love.graphics.setBackgroundColor(0, 0, 0)
	love.graphics.setMode(650, 650, false, true, 0)
end

function handleKeyboard()
	if love.keyboard.isDown("right") then
		player.body:applyForce(800, 0)
	elseif love.keyboard.isDown("left") then
		player.body:applyForce(-800, 0)
	end
end

function updateCamera()
	--Useful legibility vars.
	local p_x = player.body:getX()
	local p_y = player.body:getY()
	local e_x = enemy.body:getX()
	local e_y = enemy.body:getY()

	--Zoom it out based on distance.
	local x = p_x - e_x
	local y = p_y - e_y
	local dist = math.sqrt(x*x + y*y)
	local pad = 0.45
	local scale = math.log(math.abs(dist))*pad
	camera:setScale(scale,scale)

	--Make camera look at the center between the two balls.
	local mid_point_x = (p_x + e_x)/2
	local mid_point_y = (p_y + p_x)/2
	local offset_x = W/2*scale
	local offset_y = H/2*scale
	camera:setPosition(mid_point_x-offset_x, mid_point_y-offset_y)
end

function love.update(dt)
	--Move camera
	updateCamera()

	world:update(dt)
	handleKeyboard()

end

function love.draw()
	camera:set()
	for key, ball in pairs(objects.balls) do
		love.graphics.setColor(ball.color[1],ball.color[2],ball.color[3])
		love.graphics.circle("line", ball.body:getX(), ball.body:getY(), ball.shape:getRadius())
	end
	love.graphics.setColor(255,255,255)
	for key, platform in pairs(objects.platforms) do
		love.graphics.polygon("fill", platform.body:getWorldPoints(platform.shape:getPoints()))
	end
	for key,wall in pairs(objects.walls) do
		love.graphics.polygon("fill", wall.body:getWorldPoints(wall.shape:getPoints()))
	end
	camera:unset()
end
